<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2007 mehrwert (typo3@mehrwert.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   73: class SC_mod_web_perm_ajax
 *
 *              SECTION: Init method for this class
 *  108:     function init( $backPath = '' )
 *
 *              SECTION: Main dispatcher method
 *  157:     function main()
 *
 *              SECTION: Helpers for this script
 *  275:     function printUserSelector($page, $owner_uid, $username = '')
 *  319:     function printGroupSelector($page, $group_uid, $groupname = '')
 *  369:     function printChangedOwner($page, $owner_uid, $username)
 *  382:     function printChangedGroup($page, $group_uid, $groupname)
 *  394:     function printToggleEditLock($page, $editlockstate)
 *  408:     function printPerms($int, $pageId = 0, $who = 'user')
 *  423:     function printContent()
 *
 * TOTAL FUNCTIONS: 9
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */

// Unset the MCONF array
unset($MCONF);

// Basic includes (and proper authentication)
require_once 'conf.php';
require_once $BACK_PATH.'init.php';
require_once $BACK_PATH.'template.php';
require_once PATH_t3lib.'class.t3lib_tcemain.php';

//$LANG->includeLLFile('EXT:lang/locallang_mod_web_perm.xml');
$BE_USER->modAccess($MCONF, 1);

/**
 * This class extends the permissions module in the TYPO3 Backend to provide convenient methods of
 * editing of page permissions (including page ownership (user and group)) via AJAX
 *
 * @author		Andreas Kundoch <typo3@mehrwert.de>
 * @version		$Id: mod_perm_ajax_responder.php 3027 2007-09-02 12:23:28Z ab $
 * @package		TYPO3
 * @subpackage	core
 * @license		GPL
 */
class SC_mod_web_perm_ajax {

	/**
	 * The local configuration array
	 *
	 * @var	Array
	 */
	var $config = array();

	/**
	 * The return value of the class
	 *
	 * @var	String
	 */
	var $content = '';

	/**
	 * TYPO3 Back Path
	 *
	 * @var	String
	 */
	var $backPath = '';

	/********************************************
	 *
	 * Init method for this class
	 *
	 ********************************************/

	/**
	 * The init method of this class
	 *
	 * @param	String		$backPath: TYPO3 Back Path
	 * @return	Void
	 */
	function init( $backPath = '' ) {

		// Set the back path
		$this->backPath = $backPath;

		// Configuration, variable assignment
		$this->config['page'] 			= intval($_POST['page']);
		$this->config['who'] 			= $_POST['who'];
		$this->config['mode'] 			= $_POST['mode'];
		$this->config['bits'] 			= intval($_POST['bits']);
		$this->config['permissions'] 	= intval($_POST['permissions']);
		$this->config['action']			= $_POST['action'];
		$this->config['owner_uid']		= intval($_POST['owner_uid']);
		$this->config['username']		= $_POST['username'];
		$this->config['group_uid']		= intval($_POST['group_uid']);
		$this->config['groupname']		= $_POST['groupname'];
		$this->config['editlockstate']	= intval($_POST['editlockstate']);

		// User: Replace some parts of the posted values
		$this->config['owner_data']		= urldecode($_POST['owner_data']);
		$this->config['owner_data']		= str_replace('new_page_owner=', '', $this->config['owner_data']);
		$this->config['owner_data']		= str_replace('%3B', ';', $this->config['owner_data']);

		$temp_owner_data 					= explode(';', $this->config['owner_data']);
		$this->config['new_owner_uid'] 		= intval($temp_owner_data[0]);
		$this->config['new_owner_username'] = htmlspecialchars($temp_owner_data[1]);

		// Group: Replace some parts of the posted values
		$this->config['group_data']	= urldecode($_POST['group_data']);
		$this->config['group_data']	= str_replace('new_page_group=', '', $this->config['group_data']);
		$this->config['group_data']	= str_replace('%3B', ';', $this->config['group_data']);

		$temp_group_data 					= explode(';', $this->config['group_data']);
		$this->config['new_group_uid'] 		= intval($temp_group_data[0]);
		$this->config['new_group_username'] = htmlspecialchars($temp_group_data[1]);

	}

	/********************************************
	 *
	 * Main dispatcher method
	 *
	 ********************************************/

	/**
	 * The main function. Collect data and prepare HTML output.
	 *
	 * @return	Void
	 */
	function main() {

		// Basic test for required value
		if ( is_int($this->config['page']) ) {

			// Determine the scripts to execute
			switch ( $this->config['action'] ) {
				// Return the select to change the owner (BE user) of the page
				case 'show_change_owner_selector':
					$this->content = $this->printUserSelector($this->config['page'], $this->config['owner_uid'], $this->config['username']);
					break;

				// Change the owner and return the new owner HTML snippet
				case 'change_owner':
					if ( is_int($this->config['new_owner_uid']) ) {

						// Prepare data to change
						$data = array();
						$data['pages'][$this->config['page']]['perms_userid'] = $this->config['new_owner_uid'];

						// Init TCE and execute update
						$tce = t3lib_div::makeInstance('t3lib_TCEmain');
						$tce->stripslashes_values = 1;
						$tce->start($data, array());
						$tce->process_datamap();

						$this->content = $this->printChangedOwner($this->config['page'], $this->config['new_owner_uid'], $this->config['new_owner_username']);
					}

					else {
						$this->content = 'An error occured: No page owner uid specified.';
					}
					break;

				// Return the select to change the group (BE group) of the page
				case 'show_change_group_selector':
					$this->content = $this->printgroupSelector($this->config['page'], $this->config['group_uid'], $this->config['groupname']);
					break;

				// Change the group and return the new group HTML snippet
				case 'change_group':
					if ( is_int($this->config['new_group_uid']) ) {

						// Prepare data to change
						$data = array();
						$data['pages'][$this->config['page']]['perms_groupid'] = $this->config['new_group_uid'];

						// Init TCE and execute update
						$tce = t3lib_div::makeInstance('t3lib_TCEmain');
						$tce->stripslashes_values = 1;
						$tce->start($data, array());
						$tce->process_datamap();

						$this->content = $this->printChangedGroup($this->config['page'], $this->config['new_group_uid'], $this->config['new_group_username']);
					}
					else {
						$this->content = 'An error occured: No page group uid specified.';
					}
					break;

				// Change the group and return the new group HTML snippet
				case 'toggle_edit_lock':

					// Prepare data to change
					$data = array();
					$data['pages'][$this->config['page']]['editlock'] = ( $this->config['editlockstate'] === 1 ? 0 : 1 );

					// Init TCE and execute update
					$tce = t3lib_div::makeInstance('t3lib_TCEmain');
					$tce->stripslashes_values = 1;
					$tce->start($data, array());
					$tce->process_datamap();

					$this->content = $this->printToggleEditLock($this->config['page'], $data['pages'][$this->config['page']]['editlock']);
					break;

				// The script defaults to change permissions
				default:
					if ( $this->config['mode'] == 'delete' ) {
						$this->config['permissions'] = intval($this->config['permissions'] - $this->config['bits']);
					}
					else {
						$this->config['permissions'] = intval($this->config['permissions'] + $this->config['bits']);
					}

					// Prepare data to change
					$data = array();
					$data['pages'][$this->config['page']]['perms_'.$this->config['who']] = $this->config['permissions'];

					// Init TCE and execute update
					$tce = t3lib_div::makeInstance('t3lib_TCEmain');
					$tce->stripslashes_values = 1;
					$tce->start($data, array());
					$tce->process_datamap();

					$this->content = $this->printPerms($this->config['permissions'], $this->config['page'], $this->config['who']);
					break;
			}
		}
		else {
			$this->content = 'This script cannot be called directly.';
		}
	}

	/********************************************
	 *
	 * Helpers for this script
	 *
	 ********************************************/

	/**
	 * Generate the user selector element
	 *
	 * @param	Integer		$page: The page id to change the user for
	 * @param	Integer		$owner_uid: The page owner uid
	 * @param	String		$username: The username to display
	 * @return	String		The html select element
	 */
	function printUserSelector($page, $owner_uid, $username = '') {

		// Get usernames
		$be_user_Array = t3lib_BEfunc::getUserNames();

		// Init groupArray
		$groupArray = array();

		if ( !$GLOBALS['BE_USER']->isAdmin() ) {
			$be_user_Array = t3lib_BEfunc::blindUserNames($be_user_Array, $groupArray, 1);
		}

		// Owner selector:
		$options = '';

		// Loop through the users
		foreach ( $be_user_Array AS $uid => $row )	{
			if ( $uid == $owner_uid )	{
				$selected = ' selected="selected"';
			}
			else {
				$selected = '';
			}
			$options .= '<option value="'.$uid.';'.htmlspecialchars($row['username']).'"'.$selected.'>'.htmlspecialchars($row['username']).'</option>';
		}

		$options = '<option value="0"></option>'.$options;
		$selector = '<select name="new_page_owner" id="new_page_owner">'.$options.'</select>';
		$saveButton = '<a onClick="WebPermissions.changeOwner(\''.$page.'\', \''.$owner_uid.'\', \'o_'.$page.'_'.$owner_uid.'\');"><img'.t3lib_iconWorks::skinImg($this->backPath, 'gfx/savedok.gif', 'width="21" height="16"').' border="0" title="Change owner" align="top" alt="" /></a>';
		$cancelButton = '<a onClick="WebPermissions.restoreOwner(\''.$page.'\', \''.$owner_uid.'\', \''.$username.( $username == '' ? '<strong><i>[not set]</i></strong>' : '' ).'\', \'o_'.$page.'_'.$owner_uid.'\');"><img'.t3lib_iconWorks::skinImg($this->backPath, 'gfx/closedok.gif', 'width="21" height="16"').' border="0" title="Cancel" align="top" alt="" /></a>';
		$ret = '<div>'.$selector.$saveButton.$cancelButton.'</div>';

		return $ret;

	}

	/**
	 * Generate the group selector element
	 *
	 * @param	Integer		$page: The page id to change the user for
	 * @param	Integer		$group_uid: The page group uid
	 * @param	String		$username: The username to display
	 * @return	String		The html select element
	 */
	function printGroupSelector($page, $group_uid, $groupname = '') {

		$ret = '';

		// Get usernames
		$be_group_Array = t3lib_BEfunc::getListGroupNames('title,uid');
		$groupArray = array_keys($be_group_Array);
		$be_group_Array_o = $be_group_Array = t3lib_BEfunc::getGroupNames();
		if ( !$GLOBALS['BE_USER']->isAdmin() ) {
			$be_group_Array = t3lib_BEfunc::blindGroupNames($be_group_Array_o,$groupArray,1);
		}

		// Group selector:
		$options = '';
		// flag: is set if the page-groupid equals one from the group-list
		$userset = 0;

		// Loop through the groups
		foreach ($be_group_Array AS $uid => $row)	{
			if ( $uid == $group_uid )	{
				$userset = 1;
				$selected = ' selected="selected"';
			} else {
				$selected = '';
			}
			$options .= '<option value="'.$uid.';'.htmlspecialchars($row['title']).'"'.$selected.'>'.htmlspecialchars($row['title']).'</option>';
			}

			// If the group was not set AND there is a group for the page
			if (!$userset && $group_uid) {
				$options = '<option value="'.$group_uid.'" selected="selected">'.htmlspecialchars($be_group_Array_o[$group_uid]['title']).'</option>'.
				$options;
			}

		$options = '<option value="0"></option>'.$options;
		$selector = '<select name="new_page_group" id="new_page_group">'.$options.'</select>';
		$saveButton = '<a onClick="WebPermissions.changeGroup(\''.$page.'\', \''.$group_uid.'\', \'g_'.$page.'_'.$group_uid.'\');"><img'.t3lib_iconWorks::skinImg($this->backPath, 'gfx/savedok.gif', 'width="21" height="16"').' border="0" title="Change group" align="top" alt="" /></a>';
		$cancelButton = '<a onClick="WebPermissions.restoreGroup(\''.$page.'\', \''.$group_uid.'\', \''.$groupname.( $groupname == '' ? '<strong><i>[not set]</i></strong>' : '' ).'\', \'g_'.$page.'_'.$group_uid.'\');"><img'.t3lib_iconWorks::skinImg($this->backPath, 'gfx/closedok.gif', 'width="21" height="16"').' border="0" title="Cancel" align="top" alt="" /></a>';
		$ret = '<div>'.$selector.$saveButton.$cancelButton.'</div>';
		return $ret;
	}

	/**
	 * Print the string with the new owner of a page record
	 *
	 * @param	Integer		$page: The TYPO3 page id
	 * @param	Integer		$owner_uid: The new page user uid
	 * @param	String		$username: The TYPO3 BE username (used to display in the element)
	 * @return	String		The new group wrapped in HTML
	 */
	function printChangedOwner($page, $owner_uid, $username) {
		$ret = '<span id="o_'.$page.'_'.$owner_uid.'"><a class="ug_selector" onClick="WebPermissions.showChangeOwnerSelector(\''.$page.'\', \''.$owner_uid.'\', \'o_'.$page.'_'.$owner_uid.'\', \''.$username.'\');">'.$username.( $username == '' ? '<strong><i>[not set]</i></strong>' : '' ).'</a></span>';
		return $ret;
	}

	/**
	 * Print the string with the new group of a page record
	 *
	 * @param	Integer		$page: The TYPO3 page id
	 * @param	Integer		$group_uid: The new page group uid
	 * @param	String		$groupname: The TYPO3 BE groupname (used to display in the element)
	 * @return	String		The new group wrapped in HTML
	 */
	function printChangedGroup($page, $group_uid, $groupname) {
		$ret = '<span id="g_'.$page.'_'.$group_uid.'"><a class="ug_selector" onClick="WebPermissions.showChangeGroupSelector(\''.$page.'\', \''.$group_uid.'\', \'g_'.$page.'_'.$group_uid.'\', \''.$groupname.'\');">'.$groupname.( $groupname == '' ? '<strong><i>[not set]</i></strong>' : '' ).'</a></span>';
		return $ret;
	}

	/**
	 * Print the string with the new edit lock state of a page record
	 *
	 * @param	Integer		$page: The TYPO3 page id
	 * @param	String		$editlockstate: The state of the TYPO3 page (locked, unlocked)
	 * @return	String		The new edit lock string wrapped in HTML
	 */
	function printToggleEditLock($page, $editlockstate) {
		$ret = ( $editlockstate === 1 ? '<a class="editlock" onClick="WebPermissions.toggleEditLock(\''.$page.'\', \'1\');"><img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/recordlock_warning2.gif','width="22" height="16"').' title="The page and all content is locked for editing by all non-Admin users." alt="" /></a>' : '<a class="editlock" onClick="WebPermissions.toggleEditLock(\''.$page.'\', \'0\');" title="Enable the &raquo;Admin-only&laquo; edit lock for this page">[+]</a>');
		return $ret;
	}

	/**
	 * Print a set of permissions. Derived from the original method in index.php
	 *
	 * @param	integer		Permission integer (bits)
	 * @param	Integer		$page: The TYPO3 page id
	 * @param	String		$who: The scope (user, group or everybody)
	 * @return	string		HTML marked up x/* indications.
	 * @see		mod/web/perm/index.php::printPerms()
	 */
	function printPerms($int, $pageId = 0, $who = 'user') {
		$str  = '';
		$str .= ( ($int&1)  ? '<span class="perm-allowed"><a class="perm-allowed" onClick="WebPermissions.setPermissions('.$pageId.', 1, \'delete\', \''.$who.'\', \''.$int.'\');">*</a></span>'  : '<span class="perm-denied"><a class="perm-denied" onClick="WebPermissions.setPermissions('.$pageId.', 1, \'add\', \''.$who.'\', \''.$int.'\');">x</a></span>');
		$str .= ( ($int&16) ? '<span class="perm-allowed"><a class="perm-allowed" onClick="WebPermissions.setPermissions('.$pageId.', 16, \'delete\', \''.$who.'\', \''.$int.'\');">*</a></span>' : '<span class="perm-denied"><a class="perm-denied" onClick="WebPermissions.setPermissions('.$pageId.', 16, \'add\', \''.$who.'\', \''.$int.'\');">x</a></span>');
		$str .= ( ($int&2)  ? '<span class="perm-allowed"><a class="perm-allowed" onClick="WebPermissions.setPermissions('.$pageId.', 2, \'delete\', \''.$who.'\', \''.$int.'\');">*</a></span>'  : '<span class="perm-denied"><a class="perm-denied" onClick="WebPermissions.setPermissions('.$pageId.', 2, \'add\', \''.$who.'\', \''.$int.'\');">x</a></span>');
		$str .= ( ($int&4)  ? '<span class="perm-allowed"><a class="perm-allowed" onClick="WebPermissions.setPermissions('.$pageId.', 4, \'delete\', \''.$who.'\', \''.$int.'\');">*</a></span>'  : '<span class="perm-denied"><a class="perm-denied" onClick="WebPermissions.setPermissions('.$pageId.', 4, \'add\', \''.$who.'\', \''.$int.'\');">x</a></span>');
		$str .= ( ($int&8)  ? '<span class="perm-allowed"><a class="perm-allowed" onClick="WebPermissions.setPermissions('.$pageId.', 8, \'delete\', \''.$who.'\', \''.$int.'\');">*</a></span>'  : '<span class="perm-denied"><a class="perm-denied" onClick="WebPermissions.setPermissions('.$pageId.', 8, \'add\', \''.$who.'\', \''.$int.'\');">x</a></span>');
		return $str;
	}

	/**
	 * Outputting the accumulated content to screen
	 *
	 * @return	void
	 */
	function printContent() {
		echo $this->content;
	}
}

// Include extension?
if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['typo3/mod/web/perm/mod_perm_ajax_responder.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['typo3/mod/web/perm/mod_perm_ajax_responder.php.php']);
}

// Make instance:
$SOBE = t3lib_div::makeInstance('SC_mod_web_perm_ajax');
$SOBE->init($BACK_PATH);
$SOBE->main();
$SOBE->printContent();

if ($TYPO3_CONF_VARS['BE']['compressionLevel'])	{
	new gzip_encode($TYPO3_CONF_VARS['BE']['compressionLevel']);
}

?>